/*
 * xmalloc.c
 *
 * This file is part of the msmtp package, a very simple programm that 
 * works as an "SMTP plugin" for Mutt and possibly other MUAs.
 *
 * Copyright (C) 2000, 2003	Martin Lambers <marlam@web.de>
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <string.h>

#include "merror.h"


/* xmalloc() ***************************************************************/

void *xmalloc(size_t size)
{
    void *ptr;

    if (!(ptr = malloc(size))) 
    {
	die("xmalloc() failed!");
    }
    return ptr;
}


/* xcalloc() ***************************************************************/

void *xcalloc(size_t n, size_t size)
{
    void *ptr;

    if (!(ptr = calloc(n, size)))
    {
	die("xcalloc() failed!");
    }
    return ptr;
}

    
/* xrealloc() **************************************************************/

void *xrealloc(void *ptr, size_t newsize)
{
    if (!(ptr = realloc(ptr, newsize)))
    {
	die("xrealloc() failed!");
    }
    return ptr;
}


/* xstrdup() ***************************************************************/

char *xstrdup(const char *s)
{
    char *p;

    if (!(p = strdup(s)))
    {
	die("xstrdup() failed!");
    }
    return p;
}


/* xvasprintf() ************************************************************/

int xvasprintf(char **strp, const char *format, va_list ap)
{
    int count;
#ifndef HAVE_VASPRINTF
    size_t size;
    size = 256;
    for (;;)
    {
	*strp = (char *)xmalloc(size * sizeof(char));
	count = vsnprintf(*strp, size, format, ap);
	if (count < size)
	{
	    break;
	}
	else
	{
	    free(*strp);
	    size += 256;
	}
    }
#else
    count = vasprintf(strp, format, ap);
    if (!*strp)
    {
	die("vasprintf failed!");
    }
#endif /* not HAVE_VASPRINTF */
    return count;
}


/* xasprintf() *************************************************************/

int xasprintf(char **strp, const char *format, ...)
{
    int count;
    va_list args;
    
    va_start(args, format);
    count = xvasprintf(strp, format, args);
    va_end(args);
    
    return count;
}
